/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef COLOR_HH
#define COLOR_HH

#include <gdkmm/color.h>

namespace SharpConstruct
{
	class Color
	{
	public:
		Color();
		Color( const Color& );
		Color( const float, const float, const float );
		Color( const float, const float, const float, const float );
		Color( const Gdk::Color& );

		Gdk::Color GdkColor() const;

		float R() const;
		float G() const;
		float B() const;
		float Mask() const;

		const float* Data() const;
		float* Data();

		void Set( const float, const float, const float, const float );
		void SetRGB( const float, const float, const float );
		void SetR( const float );
		void SetG( const float );
		void SetB( const float );
		void SetMask( const float );

		void Clamp( float& ) const;
		
		void operator=( const Color& );
		Color operator+( const Color& ) const;
		Color operator+( const float ) const;
		Color operator-( const Color& ) const;
		Color operator-( const float ) const;
		Color operator*( const float ) const;
		void operator+=( const Color& );
		void operator/=( const int );
	private:
		float components_[ 4 ];
	};

	Color MixColors( const Color&, const Color& );
	Color MixColors( const Color&, const Color&, const Color& );
	Color MixColors( const Color&, const Color&, const Color&,
	                 const Color& );
}

#endif
