/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef DEFORMATIONBASE_HH
#define DEFORMATIONBASE_HH

#include "Point.hh"
#include "gdkmm/types.h"

namespace SharpConstruct
{
	class DeformationBase
	{
	public:
		DeformationBase();
		virtual ~DeformationBase();

		virtual void FlatDraw( const Gdk::Point&,
				       const Gdk::Point& model_center );
		virtual void Draw( const Gdk::Point& );
		virtual void ModifyModel() = 0;
		virtual void MouseDown( const Gdk::Point& );
		virtual void MouseMove( const Gdk::Point& );
		virtual void MouseUp( const Gdk::Point& ) = 0;

		virtual void Apply() = 0;
	protected:
		enum Axis{ NONE, X, Y, Z, ALL };

		virtual void draw_and_select_();
		virtual void draw_handle_( const Axis, const float x,
					   const float y, const float z ) = 0;
		Gdk::Point project_2d_( const Point3D& ) const;
		Axis over_, selected_;
		Gdk::Point model_center_;
		Gdk::Point mouse_down_loc_;
	};
}

#endif
