/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef MODELDISPLAY_HH
#define MODELDISPLAY_HH

#include "ArcBall.hh"
#include "Create.hh"
#include "Math.hh"
#include "Mesh.hh"
#include "Output3D.hh"

#ifdef WIN32
#include <windows.h>
#undef min
#undef max
#endif

#include <gtkglmm.h>

namespace SharpConstruct
{
	/* Similar to the Editor class,
	   but optimized for simple display
	   with no editing operations. */
	
	class ModelDisplay : public Gtk::GL::DrawingArea
	{
	public:
		ModelDisplay();
	       
		void SetMesh( Mesh& );
	protected:
		virtual void on_realize();
		virtual bool on_configure_event( GdkEventConfigure* );
		virtual bool on_expose_event( GdkEventExpose* );

		virtual bool on_button_press_event( GdkEventButton* );
		virtual bool on_motion_notify_event( GdkEventMotion* );
		virtual bool on_button_release_event( GdkEventButton* );
		virtual bool on_scroll_event( GdkEventScroll* );
	private:
		void set_viewport_();
		void orientate_();

		Math::Point2D< int > last_mouse_location_;

		Output3D* glout_;

		// Camera
		ArcBall arc_ball_;
		Math::Point2D< float > location_;
		float zoom_;

		Mesh* mesh_;
	};
}

#endif
