/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef SKIN_HH
#define SKIN_HH

#include "Intersection.hh"
#include "Point.hh"
#include <vector>

namespace SharpConstruct
{
	namespace Deformations
	{
		class Skin
		{
		public:
			class Node
			{
			public:
				Node()
				: Inside( false ), Checked( false ),
				  Index( 0 )
				{}

				bool Inside;
				bool Checked;
				Point3D Location;
				unsigned Index;
			};

			class Edge
			{
			public:
				Edge( Node&, Node& );

				Node* Start;
				Node* End;
				Intersections Hits;
				int Index;
			};

			void operator()( const unsigned );
		private:
			/* Finds a grid formed of nodes and edges.
			 *  
			 * Each node is a signed (inside vs. outside) grid
			 * vertex. Edges connect the nodes to form cells
			 * (cube shaped) in the grid. Edges also store any
			 * intersections the have with the model's surface.
			 * This "hermite data" can be used as the input for
			 * any of the marching cubes algorithms.
			 */
			void find_grid_();
			

			void simple_mc_();

			Node& get_node_( const unsigned, const unsigned,
					 const unsigned );
			Edge& get_edge_( const char, const unsigned,
					 const unsigned, const unsigned );

			std::vector< Node > nodes_;
			std::vector< Edge > edges_x_, edges_y_, edges_z_;
			unsigned divs_;
		};
	}
}

#endif
